@@ -35,6 +35,7 @@
 require 'rbconfig'
 require 'find'
 require 'fileutils'
+require 'tempfile'
 begin
   require 'ftools' # apparently on some system ftools doesn't get loaded
   $haveftools = true
@@ -226,8 +227,6 @@ def prepare_installation
     opts.parse!
   end
 
-  tmpdirs = [ENV['TMP'], ENV['TEMP'], "/tmp", "/var/tmp", "."]
-
   version = [Config::CONFIG["MAJOR"], Config::CONFIG["MINOR"]].join(".")
   libdir = File.join(Config::CONFIG["libdir"], "ruby", version)
 
@@ -310,9 +309,6 @@ def prepare_installation
   FileUtils.makedirs(mandir)
   FileUtils.makedirs(sitelibdir)
 
-  tmpdirs << bindir
-
-  InstallOptions.tmp_dirs = tmpdirs.compact
   InstallOptions.site_dir = sitelibdir
   InstallOptions.config_dir = configdir
   InstallOptions.bin_dir  = bindir
@@ -385,20 +381,11 @@ end
 # (e.g., bin/rdoc becomes rdoc); the shebang line handles running it. Under
 # windows, we add an '.rb' extension and let file associations do their stuff.
 def install_binfile(from, op_file, target)
-  tmp_dir = nil
-  InstallOptions.tmp_dirs.each do |t|
-    if File.directory?(t) and File.writable?(t)
-      tmp_dir = t
-      break
-    end
-  end
-
-  fail "Cannot find a temporary directory" unless tmp_dir
-  tmp_file = File.join(tmp_dir, '_tmp')
+  tmp_file = Tempfile.new('puppet-binfile')
   ruby = File.join(Config::CONFIG['bindir'], Config::CONFIG['ruby_install_name'])
 
   File.open(from) do |ip|
-    File.open(tmp_file, "w") do |op|
+    File.open(tmp_file.path, "w") do |op|
       ruby = File.join(Config::CONFIG['bindir'], Config::CONFIG['ruby_install_name'])
       op.puts "#!#{ruby}"
       contents = ip.readlines
@@ -421,7 +408,7 @@ def install_binfile(from, op_file, target)
     end
 
     if not installed_wrapper
-      tmp_file2 = File.join(tmp_dir, '_tmp_wrapper')
+      tmp_file2 = Tempfile.new('puppet-wrapper')
       cwv = <<-EOS
 @echo off
 setlocal
@@ -429,15 +416,15 @@ set RUBY_BIN=%~dp0
 set RUBY_BIN=%RUBY_BIN:\\=/%
 "%RUBY_BIN%ruby.exe" -x "%RUBY_BIN%puppet" %*
 EOS
-      File.open(tmp_file2, "w") { |cw| cw.puts cwv }
-      FileUtils.install(tmp_file2, File.join(target, "#{op_file}.bat"), :mode => 0755, :verbose => true)
+      File.open(tmp_file2.path, "w") { |cw| cw.puts cwv }
+      FileUtils.install(tmp_file2.path, File.join(target, "#{op_file}.bat"), :mode => 0755, :verbose => true)
 
-      File.unlink(tmp_file2)
+      tmp_file2.unlink
       installed_wrapper = true
     end
   end
-  FileUtils.install(tmp_file, File.join(target, op_file), :mode => 0755, :verbose => true)
-  File.unlink(tmp_file)
+  FileUtils.install(tmp_file.path, File.join(target, op_file), :mode => 0755, :verbose => true)
+  tmp_file.unlink
 end
 
 check_prereqs
