@@ -2,7 +2,7 @@ module Puppet::Module::Tool
   module Applications
     class Cleaner < Application
       def run
-        Puppet::Module::Tool::Cache.clean
+        Puppet::Forge::Cache.clean
 
         # Return a status Hash containing the status of the clean command
         # and a status message. This return value is used by the module_tool
