@@ -5,10 +5,12 @@ require 'puppet/indirector/catalog/rest'
 require 'tempfile'
 
 describe Puppet::Face[:secret_agent, '0.0.1'] do
+  include PuppetSpec::Files
+
   describe "#synchronize" do
     it "should retrieve and apply a catalog and return a report" do
       pending "This test doesn't work, but the code actually does - tested by LAK"
-      dirname = Dir.mktmpdir("puppetdir")
+      dirname = tmpdir("puppetdir")
       Puppet[:vardir] = dirname
       Puppet[:confdir] = dirname
       @catalog = Puppet::Resource::Catalog.new
