@@ -4,6 +4,8 @@ require 'spec_helper'
 require 'puppet/file_serving/metadata'
 
 describe Puppet::FileServing::Metadata do
+  let(:foobar) { File.expand_path('/foo/bar') }
+
   it "should should be a subclass of Base" do
     Puppet::FileServing::Metadata.superclass.should equal(Puppet::FileServing::Base)
   end
@@ -17,15 +19,15 @@ describe Puppet::FileServing::Metadata do
   end
 
   it "should have a method that triggers attribute collection" do
-    Puppet::FileServing::Metadata.new("/foo/bar").should respond_to(:collect)
+    Puppet::FileServing::Metadata.new(foobar).should respond_to(:collect)
   end
 
   it "should support pson serialization" do
-    Puppet::FileServing::Metadata.new("/foo/bar").should respond_to(:to_pson)
+    Puppet::FileServing::Metadata.new(foobar).should respond_to(:to_pson)
   end
 
   it "should support to_pson_data_hash" do
-    Puppet::FileServing::Metadata.new("/foo/bar").should respond_to(:to_pson_data_hash)
+    Puppet::FileServing::Metadata.new(foobar).should respond_to(:to_pson_data_hash)
   end
 
   it "should support pson deserialization" do
@@ -33,67 +35,66 @@ describe Puppet::FileServing::Metadata do
   end
 
   describe "when serializing" do
-    before do
-      @metadata = Puppet::FileServing::Metadata.new("/foo/bar")
-    end
+    let(:metadata) { Puppet::FileServing::Metadata.new(foobar) }
+
     it "should perform pson serialization by calling to_pson on it's pson_data_hash" do
       pdh = mock "data hash"
       pdh_as_pson = mock "data as pson"
-      @metadata.expects(:to_pson_data_hash).returns pdh
+      metadata.expects(:to_pson_data_hash).returns pdh
       pdh.expects(:to_pson).returns pdh_as_pson
-      @metadata.to_pson.should == pdh_as_pson
+      metadata.to_pson.should == pdh_as_pson
     end
 
     it "should serialize as FileMetadata" do
-      @metadata.to_pson_data_hash['document_type'].should == "FileMetadata"
+      metadata.to_pson_data_hash['document_type'].should == "FileMetadata"
     end
 
     it "the data should include the path, relative_path, links, owner, group, mode, checksum, type, and destination" do
-      @metadata.to_pson_data_hash['data'].keys.sort.should == %w{ path relative_path links owner group mode checksum type destination }.sort
+      metadata.to_pson_data_hash['data'].keys.sort.should == %w{ path relative_path links owner group mode checksum type destination }.sort
     end
 
     it "should pass the path in the hash verbatum" do
-      @metadata.to_pson_data_hash['data']['path'] == @metadata.path
+      metadata.to_pson_data_hash['data']['path'] == metadata.path
     end
 
     it "should pass the relative_path in the hash verbatum" do
-      @metadata.to_pson_data_hash['data']['relative_path'] == @metadata.relative_path
+      metadata.to_pson_data_hash['data']['relative_path'] == metadata.relative_path
     end
 
     it "should pass the links in the hash verbatum" do
-      @metadata.to_pson_data_hash['data']['links'] == @metadata.links
+      metadata.to_pson_data_hash['data']['links'] == metadata.links
     end
 
     it "should pass the path owner in the hash verbatum" do
-      @metadata.to_pson_data_hash['data']['owner'] == @metadata.owner
+      metadata.to_pson_data_hash['data']['owner'] == metadata.owner
     end
 
     it "should pass the group in the hash verbatum" do
-      @metadata.to_pson_data_hash['data']['group'] == @metadata.group
+      metadata.to_pson_data_hash['data']['group'] == metadata.group
     end
 
     it "should pass the mode in the hash verbatum" do
-      @metadata.to_pson_data_hash['data']['mode'] == @metadata.mode
+      metadata.to_pson_data_hash['data']['mode'] == metadata.mode
     end
 
     it "should pass the ftype in the hash verbatum as the 'type'" do
-      @metadata.to_pson_data_hash['data']['type'] == @metadata.ftype
+      metadata.to_pson_data_hash['data']['type'] == metadata.ftype
     end
 
     it "should pass the destination verbatum" do
-      @metadata.to_pson_data_hash['data']['destination'] == @metadata.destination
+      metadata.to_pson_data_hash['data']['destination'] == metadata.destination
     end
 
     it "should pass the checksum in the hash as a nested hash" do
-      @metadata.to_pson_data_hash['data']['checksum'].should be_is_a(Hash)
+      metadata.to_pson_data_hash['data']['checksum'].should be_is_a(Hash)
     end
 
     it "should pass the checksum_type in the hash verbatum as the checksum's type" do
-      @metadata.to_pson_data_hash['data']['checksum']['type'] == @metadata.checksum_type
+      metadata.to_pson_data_hash['data']['checksum']['type'] == metadata.checksum_type
     end
 
     it "should pass the checksum in the hash verbatum as the checksum's value" do
-      @metadata.to_pson_data_hash['data']['checksum']['value'] == @metadata.checksum
+      metadata.to_pson_data_hash['data']['checksum']['value'] == metadata.checksum
     end
 
   end
