@@ -1,9 +1,9 @@
-#!/usr/bin/env ruby
+#!/usr/bin/env rspec
 #
 # Unit testing for the Init service Provider
 #
 
-require File.dirname(__FILE__) + '/../../../spec_helper'
+require 'spec_helper'
 
 provider_class = Puppet::Type.type(:service).provider(:init)
 
@@ -46,7 +46,7 @@ describe provider_class do
       results = (@services-exclude).collect {|x| "#{x}_instance"}
       @class.get_services(@class.defpath, exclude).should == results
     end
-    it "should omit a single service from the exclude list" do
+    it "should omit a single service from the exclude list", :'fails_on_ruby_1.9.2' => true do
       exclude = 'two'
       (@services-exclude.to_a).each do |inst|
         @class.expects(:new).with{|hash| hash[:name] == inst}.returns("#{inst}_instance")
