@@ -177,8 +177,21 @@ describe Puppet::Util do
 
   describe "execution methods" do
     let(:pid) { 5501 }
+    let(:process_handle) { 0xDEADBEEF }
+    let(:thread_handle) { 0xCAFEBEEF }
+    let(:proc_info_stub) { stub 'processinfo', :process_handle => process_handle, :thread_handle => thread_handle, :process_id => pid}
     let(:null_file) { Puppet.features.microsoft_windows? ? 'NUL' : '/dev/null' }
 
+    def stub_process_wait(exitstatus)
+      if Puppet.features.microsoft_windows?
+        Puppet::Util::Windows::Process.stubs(:wait_process).with(process_handle).returns(exitstatus)
+        Process.stubs(:CloseHandle).with(process_handle)
+        Process.stubs(:CloseHandle).with(thread_handle)
+      else
+        Process.stubs(:waitpid2).with(pid).returns([pid, stub('child_status', :exitstatus => exitstatus)])
+      end
+    end
+
     describe "#execute_posix" do
       before :each do
         # Most of the things this method does are bad to do during specs. :/
@@ -251,12 +264,10 @@ describe Puppet::Util do
       end
     end
 
-    describe "#execute_windows" do
-      let(:proc_info_stub) { stub 'processinfo', :process_id => pid }
-
+    describe "#execute_windows", :if => Puppet.features.microsoft_windows? do
       before :each do
         Process.stubs(:create).returns(proc_info_stub)
-        Process.stubs(:waitpid2).with(pid).returns([pid, process_status(0)])
+        stub_process_wait(0)
 
         @stdin  = File.open(null_file, 'r')
         @stdout = Tempfile.new('stdout')
@@ -266,14 +277,15 @@ describe Puppet::Util do
       it "should create a new process for the command" do
         Process.expects(:create).with(
           :command_line => "test command",
-          :startup_info => {:stdin => @stdin, :stdout => @stdout, :stderr => @stderr}
+          :startup_info => {:stdin => @stdin, :stdout => @stdout, :stderr => @stderr},
+          :close_handles => false
         ).returns(proc_info_stub)
 
         Puppet::Util.execute_windows('test command', {}, @stdin, @stdout, @stderr)
       end
 
-      it "should return the pid of the child process" do
-        Puppet::Util.execute_windows('test command', {}, @stdin, @stdout, @stderr).should == pid
+      it "should return the process info of the child process" do
+        Puppet::Util.execute_windows('test command', {}, @stdin, @stdout, @stderr).should == proc_info_stub
       end
 
       it "should quote arguments containing spaces if command is specified as an array" do
@@ -287,7 +299,7 @@ describe Puppet::Util do
 
     describe "#execute" do
       before :each do
-        Process.stubs(:waitpid2).with(pid).returns([pid, process_status(0)])
+        stub_process_wait(0)
       end
 
       describe "when an execution stub is specified" do
@@ -312,6 +324,7 @@ describe Puppet::Util do
       describe "when setting up input and output files" do
         include PuppetSpec::Files
         let(:executor) { Puppet.features.microsoft_windows? ? 'execute_windows' : 'execute_posix' }
+        let(:rval) { Puppet.features.microsoft_windows? ? proc_info_stub : pid }
 
         before :each do
           Puppet::Util.stubs(:wait_for_output)
@@ -323,7 +336,7 @@ describe Puppet::Util do
 
           Puppet::Util.expects(executor).with do |_,_,stdin,_,_|
             stdin.path == input
-          end.returns(pid)
+          end.returns(rval)
 
           Puppet::Util.execute('test command', :stdinfile => input)
         end
@@ -331,7 +344,7 @@ describe Puppet::Util do
         it "should set stdin to the null file if not specified" do
           Puppet::Util.expects(executor).with do |_,_,stdin,_,_|
             stdin.path == null_file
-          end.returns(pid)
+          end.returns(rval)
 
           Puppet::Util.execute('test command')
         end
@@ -340,7 +353,7 @@ describe Puppet::Util do
           it "should set stdout and stderr to the null file" do
             Puppet::Util.expects(executor).with do |_,_,_,stdout,stderr|
               stdout.path == null_file and stderr.path == null_file
-            end.returns(pid)
+            end.returns(rval)
 
             Puppet::Util.execute('test command', :squelch => true)
           end
@@ -353,7 +366,7 @@ describe Puppet::Util do
 
             Puppet::Util.expects(executor).with do |_,_,_,stdout,_|
               stdout.path == outfile.path
-            end.returns(pid)
+            end.returns(rval)
 
             Puppet::Util.execute('test command', :squelch => false)
           end
@@ -364,7 +377,7 @@ describe Puppet::Util do
 
             Puppet::Util.expects(executor).with do |_,_,_,stdout,stderr|
               stdout.path == outfile.path and stderr.path == outfile.path
-            end.returns(pid)
+            end.returns(rval)
 
             Puppet::Util.execute('test command', :squelch => false, :combine => true)
           end
@@ -375,28 +388,40 @@ describe Puppet::Util do
 
             Puppet::Util.expects(executor).with do |_,_,_,stdout,stderr|
               stdout.path == outfile.path and stderr.path == null_file
-            end.returns(pid)
+            end.returns(rval)
 
             Puppet::Util.execute('test command', :squelch => false, :combine => false)
           end
         end
       end
+
+      describe "on Windows", :if => Puppet.features.microsoft_windows? do
+        it "should always close the process and thread handles" do
+          Puppet::Util.stubs(:execute_windows).returns(proc_info_stub)
+
+          Puppet::Util::Windows::Process.expects(:wait_process).with(process_handle).raises('whatever')
+          Process.expects(:CloseHandle).with(thread_handle)
+          Process.expects(:CloseHandle).with(process_handle)
+
+          expect { Puppet::Util.execute('test command') }.should raise_error(RuntimeError)
+        end
+      end
     end
 
     describe "after execution" do
-      let(:executor) { Puppet.features.microsoft_windows? ? 'execute_windows' : 'execute_posix' }
-
       before :each do
-        Process.stubs(:waitpid2).with(pid).returns([pid, process_status(0)])
+        stub_process_wait(0)
 
-        Puppet::Util.stubs(executor).returns(pid)
+        if Puppet.features.microsoft_windows?
+          Puppet::Util.stubs(:execute_windows).returns(proc_info_stub)
+        else
+          Puppet::Util.stubs(:execute_posix).returns(pid)
+        end
       end
 
       it "should wait for the child process to exit" do
         Puppet::Util.stubs(:wait_for_output)
 
-        Process.expects(:waitpid2).with(pid).returns([pid, process_status(0)])
-
         Puppet::Util.execute('test command')
       end
 
@@ -441,7 +466,7 @@ describe Puppet::Util do
       end
 
       it "should raise an error if failonfail is true and the child failed" do
-        Process.expects(:waitpid2).with(pid).returns([pid, process_status(1)])
+        stub_process_wait(1)
 
         expect {
           Puppet::Util.execute('fail command', :failonfail => true)
@@ -449,7 +474,7 @@ describe Puppet::Util do
       end
 
       it "should not raise an error if failonfail is false and the child failed" do
-        Process.expects(:waitpid2).with(pid).returns([pid, process_status(1)])
+        stub_process_wait(1)
 
         expect {
           Puppet::Util.execute('fail command', :failonfail => false)
@@ -457,8 +482,6 @@ describe Puppet::Util do
       end
 
       it "should not raise an error if failonfail is true and the child succeeded" do
-        Process.expects(:waitpid2).with(pid).returns([pid, process_status(0)])
-
         expect {
           Puppet::Util.execute('fail command', :failonfail => true)
         }.not_to raise_error
@@ -466,6 +489,41 @@ describe Puppet::Util do
     end
   end
 
+  describe "#execpipe" do
+    let :instance do
+      instance = Class.new.new
+      instance.send(:extend, Puppet::Util)
+      instance
+    end
+
+    it "should execute a string as a string" do
+      instance.expects(:open).with('| echo hello 2>&1').returns('hello')
+      $CHILD_STATUS.expects(:==).with(0).returns(true)
+      instance.execpipe('echo hello').should == 'hello'
+    end
+
+    it "should execute an array by pasting together with spaces" do
+      instance.expects(:open).with('| echo hello 2>&1').returns('hello')
+      $CHILD_STATUS.expects(:==).with(0).returns(true)
+      instance.execpipe(['echo', 'hello']).should == 'hello'
+    end
+
+    it "should fail if asked to fail, and the child does" do
+      instance.stubs(:open).returns('error message')
+      $CHILD_STATUS.expects(:==).with(0).returns(false)
+      expect { instance.execpipe('echo hello') }.
+        to raise_error Puppet::ExecutionFailure, /error message/
+    end
+
+    it "should not fail if asked not to fail, and the child does" do
+      instance.stubs(:open).returns('error message')
+      $CHILD_STATUS.stubs(:==).with(0).returns(false)
+      expect do
+        instance.execpipe('echo hello', false).should == 'error message'
+      end.not_to raise_error
+    end
+  end
+
   describe "#which" do
     let(:base) { File.expand_path('/bin') }
     let(:path) { File.join(base, 'foo') }
